@@ -54,17 +54,22 @@ enum dump_mode {
 
 struct ctx {
 	char *device_in, *device_out, *device_trans, *filter, *prefix;
-	int cpu, rfraw, dump, print_mode, dump_dir, packet_type, verbose;
-	unsigned long kpull, dump_interval, reserve_size, tx_bytes, tx_packets;
-	bool randomize, promiscuous, enforce, jumbo, dump_bpf;
-	enum pcap_ops_groups pcap; enum dump_mode dump_mode;
-	uid_t uid; gid_t gid; uint32_t link_type, magic;
+	int cpu, rfraw, dump, print_mode, dump_dir, packet_type;
+	unsigned long kpull, dump_interval, tx_bytes, tx_packets;
+	size_t reserve_size;
+	bool randomize, promiscuous, enforce, jumbo, dump_bpf, hwtimestamp, verbose;
+	enum pcap_ops_groups pcap;
+	enum dump_mode dump_mode;
+	uid_t uid;
+	gid_t gid;
+	uint32_t link_type, magic;
+	uint32_t fanout_group, fanout_type;
 };
 
-static volatile sig_atomic_t sigint = 0;
+static volatile sig_atomic_t sigint = 0, sighup = 0;
 static volatile bool next_dump = false;
 
-static const char *short_options = "d:i:o:rf:MJt:S:k:n:b:HQmcsqXlvhF:RGAP:Vu:g:T:DBU";
+static const char *short_options = "d:i:o:rf:MNJt:S:k:n:b:HQmcsqXlvhF:RGAP:Vu:g:T:DBUC:K:L:";
 static const struct option long_options[] = {
 	{"dev",			required_argument,	NULL, 'd'},
 	{"in",			required_argument,	NULL, 'i'},
@@ -80,6 +85,9 @@ static const struct option long_options[] = {
 	{"user",		required_argument,	NULL, 'u'},
 	{"group",		required_argument,	NULL, 'g'},
 	{"magic",		required_argument,	NULL, 'T'},
+	{"fanout-group",	required_argument,	NULL, 'C'},
+	{"fanout-type",		required_argument,	NULL, 'K'},
+	{"fanout-opts",		required_argument,	NULL, 'L'},
 	{"rand",		no_argument,		NULL, 'r'},
 	{"rfraw",		no_argument,		NULL, 'R'},
 	{"mmap",		no_argument,		NULL, 'm'},
@@ -87,6 +95,7 @@ static const struct option long_options[] = {
 	{"clrw",		no_argument,		NULL, 'c'},
 	{"jumbo-support",	no_argument,		NULL, 'J'},
 	{"no-promisc",		no_argument,		NULL, 'M'},
+	{"no-hwtimestamp",	no_argument,		NULL, 'N'},
 	{"prio-high",		no_argument,		NULL, 'H'},
 	{"notouch-irq",		no_argument,		NULL, 'Q'},
 	{"dump-pcap-types",	no_argument,		NULL, 'D'},
@@ -103,6 +112,15 @@ static const struct option long_options[] = {
 	{NULL, 0, NULL, 0}
 };
 
+static const char *copyright = "Please report bugs to <bugs@netsniff-ng.org>\n"
+	"Copyright (C) 2009-2013 Daniel Borkmann <dborkma@tik.ee.ethz.ch>\n"
+	"Copyright (C) 2009-2012 Emmanuel Roullit <emmanuel.roullit@gmail.com>\n"
+	"Copyright (C) 2012 Markus Amend <markus@netsniff-ng.org>\n"
+	"Swiss federal institute of technology (ETH Zurich)\n"
+	"License: GNU GPL version 2.0\n"
+	"This is free software: you are free to change and redistribute it.\n"
+	"There is NO WARRANTY, to the extent permitted by law.";
+
 static int tx_sock;
 static struct itimerval itimer;
 static unsigned long frame_count_max = 0, interval = TX_KERNEL_PULL_INT;
@@ -116,7 +134,10 @@ static void signal_handler(int number)
 	case SIGQUIT:
 	case SIGTERM:
 		sigint = 1;
+		break;
 	case SIGHUP:
+		sighup = 1;
+		break;
 	default:
 		break;
 	}
@@ -166,11 +187,26 @@ static inline bool dump_to_pcap(struct ctx *ctx)
 	return ctx->dump;
 }
 
+static void on_panic_del_rfmon(void *arg)
+{
+	leave_rfmon_mac80211(arg);
+}
+
+static inline void setup_rfmon_mac80211_dev(struct ctx *ctx, char **rfmon_dev)
+{
+	ctx->device_trans = xstrdup(*rfmon_dev);
+	xfree(*rfmon_dev);
+
+	enter_rfmon_mac80211(ctx->device_trans, rfmon_dev);
+	panic_handler_add(on_panic_del_rfmon, *rfmon_dev);
+}
+
 static void pcap_to_xmit(struct ctx *ctx)
 {
 	uint8_t *out = NULL;
-	int irq, ifindex, fd = 0, ret;
-	unsigned int size, it = 0;
+	int ifindex, fd = 0, ret;
+	size_t size;
+	unsigned int it = 0;
 	unsigned long trunced = 0;
 	struct ring tx_ring;
 	struct frame_map *hdr;
@@ -195,7 +231,7 @@ static void pcap_to_xmit(struct ctx *ctx)
 	}
 
 	if (__pcap_io->init_once_pcap)
-		__pcap_io->init_once_pcap();
+		__pcap_io->init_once_pcap(true);
 
 	ret = __pcap_io->pull_fhdr_pcap(fd, &ctx->magic, &ctx->link_type);
 	if (ret)
@@ -207,38 +243,27 @@ static void pcap_to_xmit(struct ctx *ctx)
 			panic("Error prepare reading pcap!\n");
 	}
 
-	fmemset(&tx_ring, 0, sizeof(tx_ring));
-	fmemset(&bpf_ops, 0, sizeof(bpf_ops));
-
 	if (ctx->rfraw) {
-		ctx->device_trans = xstrdup(ctx->device_out);
-		xfree(ctx->device_out);
+		setup_rfmon_mac80211_dev(ctx, &ctx->device_out);
 
-		enter_rfmon_mac80211(ctx->device_trans, &ctx->device_out);
-		if (ctx->link_type != LINKTYPE_IEEE802_11)
+		if (ctx->link_type != LINKTYPE_IEEE802_11 &&
+				ctx->link_type != LINKTYPE_IEEE802_11_RADIOTAP)
 			panic("Wrong linktype of pcap!\n");
 	}
 
 	ifindex = device_ifindex(ctx->device_out);
-
 	size = ring_size(ctx->device_out, ctx->reserve_size);
 
 	bpf_parse_rules(ctx->filter, &bpf_ops, ctx->link_type);
 	if (ctx->dump_bpf)
 		bpf_dump_all(&bpf_ops);
 
-	set_packet_loss_discard(tx_sock);
-
-	setup_tx_ring_layout(tx_sock, &tx_ring, size, ctx->jumbo);
-	create_tx_ring(tx_sock, &tx_ring, ctx->verbose);
-	mmap_tx_ring(tx_sock, &tx_ring);
-	alloc_tx_ring_frames(tx_sock, &tx_ring);
-	bind_tx_ring(tx_sock, &tx_ring, ifindex);
+	ring_tx_setup(&tx_ring, tx_sock, size, ifindex, ctx->jumbo, ctx->verbose);
 
 	dissector_init_all(ctx->print_mode);
 
 	if (ctx->cpu >= 0 && ifindex > 0) {
-		irq = device_irq_number(ctx->device_out);
+		int irq = device_irq_number(ctx->device_out);
 		device_set_irq_affinity(irq, ctx->cpu);
 
 		if (ctx->verbose)
@@ -250,7 +275,7 @@ static void pcap_to_xmit(struct ctx *ctx)
 		interval = ctx->kpull;
 
 	set_itimer_interval_value(&itimer, 0, interval);
-	setitimer(ITIMER_REAL, &itimer, NULL); 
+	setitimer(ITIMER_REAL, &itimer, NULL);
 
 	drop_privileges(ctx->enforce, ctx->uid, ctx->gid);
 
@@ -285,10 +310,13 @@ static void pcap_to_xmit(struct ctx *ctx)
 			ctx->tx_bytes += hdr->tp_h.tp_len;;
 			ctx->tx_packets++;
 
-			show_frame_hdr(hdr, ctx->print_mode);
+			show_frame_hdr(out, hdr->tp_h.tp_snaplen,
+				       ctx->link_type, hdr, ctx->print_mode,
+				       ctx->tx_packets);
 
 			dissector_entry_point(out, hdr->tp_h.tp_snaplen,
-					      ctx->link_type, ctx->print_mode);
+					      ctx->link_type, ctx->print_mode,
+					      hdr->s_ll.sll_protocol);
 
 			kernel_may_pull_from_tx(&hdr->tp_h);
 
@@ -344,7 +372,8 @@ static void receive_to_xmit(struct ctx *ctx)
 	short ifflags = 0;
 	uint8_t *in, *out;
 	int rx_sock, ifindex_in, ifindex_out, ret;
-	unsigned int size_in, size_out, it_in = 0, it_out = 0;
+	size_t size_in, size_out;
+	unsigned int it_in = 0, it_out = 0;
 	unsigned long frame_count = 0;
 	struct frame_map *hdr_in, *hdr_out;
 	struct ring tx_ring, rx_ring;
@@ -359,11 +388,6 @@ static void receive_to_xmit(struct ctx *ctx)
 	rx_sock = pf_socket();
 	tx_sock = pf_socket();
 
-	fmemset(&tx_ring, 0, sizeof(tx_ring));
-	fmemset(&rx_ring, 0, sizeof(rx_ring));
-	fmemset(&rx_poll, 0, sizeof(rx_poll));
-	fmemset(&bpf_ops, 0, sizeof(bpf_ops));
-
 	ifindex_in = device_ifindex(ctx->device_in);
 	ifindex_out = device_ifindex(ctx->device_out);
 
@@ -377,23 +401,13 @@ static void receive_to_xmit(struct ctx *ctx)
 		bpf_dump_all(&bpf_ops);
 	bpf_attach_to_sock(rx_sock, &bpf_ops);
 
-	setup_rx_ring_layout(rx_sock, &rx_ring, size_in, ctx->jumbo, false);
-	create_rx_ring(rx_sock, &rx_ring, ctx->verbose);
-	mmap_rx_ring(rx_sock, &rx_ring);
-	alloc_rx_ring_frames(rx_sock, &rx_ring);
-	bind_rx_ring(rx_sock, &rx_ring, ifindex_in);
-	prepare_polling(rx_sock, &rx_poll);
-
-	set_packet_loss_discard(tx_sock);
-	setup_tx_ring_layout(tx_sock, &tx_ring, size_out, ctx->jumbo);
-	create_tx_ring(tx_sock, &tx_ring, ctx->verbose);
-	mmap_tx_ring(tx_sock, &tx_ring);
-	alloc_tx_ring_frames(tx_sock, &tx_ring);
-	bind_tx_ring(tx_sock, &tx_ring, ifindex_out);
+	ring_rx_setup(&rx_ring, rx_sock, size_in, ifindex_in, &rx_poll, false, ctx->jumbo,
+		      ctx->verbose, ctx->fanout_group, ctx->fanout_type);
+	ring_tx_setup(&tx_ring, tx_sock, size_out, ifindex_out, ctx->jumbo, ctx->verbose);
 
 	dissector_init_all(ctx->print_mode);
 
-	 if (ctx->promiscuous)
+	if (ctx->promiscuous)
 		ifflags = device_enter_promiscuous_mode(ctx->device_in);
 
 	if (ctx->kpull)
@@ -421,8 +435,8 @@ static void receive_to_xmit(struct ctx *ctx)
 			hdr_out = tx_ring.frames[it_out].iov_base;
 			out = ((uint8_t *) hdr_out) + TPACKET2_HDRLEN - sizeof(struct sockaddr_ll);
 
-			for (; !user_may_pull_from_tx(tx_ring.frames[it_out].iov_base) &&
-			       likely(!sigint);) {
+			while (!user_may_pull_from_tx(tx_ring.frames[it_out].iov_base) &&
+			       likely(!sigint)) {
 				if (ctx->randomize)
 					next_rnd_slot(&it_out, &tx_ring);
 				else {
@@ -447,10 +461,13 @@ static void receive_to_xmit(struct ctx *ctx)
 					it_out = 0;
 			}
 
-			show_frame_hdr(hdr_in, ctx->print_mode);
+			show_frame_hdr(in, hdr_in->tp_h.tp_snaplen,
+				       ctx->link_type, hdr_in, ctx->print_mode,
+				       frame_count);
 
 			dissector_entry_point(in, hdr_in->tp_h.tp_snaplen,
-					      ctx->link_type, ctx->print_mode);
+					      ctx->link_type, ctx->print_mode,
+					      hdr_in->s_ll.sll_protocol);
 
 			if (frame_count_max != 0) {
 				if (frame_count >= frame_count_max) {
@@ -505,7 +522,7 @@ static void translate_pcap_to_txf(int fdo, uint8_t *out, size_t len)
 	write_or_die(fdo, bout, strlen(bout));
 
 	while (bytes_done < len) {
-		slprintf(bout, sizeof(bout), "0x%02x, ", out[bytes_done]);
+		slprintf(bout, sizeof(bout), "0x%02x,", out[bytes_done]);
 		write_or_die(fdo, bout, strlen(bout));
 
 		bytes_done++;
@@ -518,6 +535,9 @@ static void translate_pcap_to_txf(int fdo, uint8_t *out, size_t len)
 				slprintf(bout, sizeof(bout), "  ");
 				write_or_die(fdo, bout, strlen(bout));
 			}
+		} else if (bytes_done < len) {
+			slprintf(bout, sizeof(bout), " ");
+			write_or_die(fdo, bout, strlen(bout));
 		}
 	}
 	if (bytes_done % 10 != 0) {
@@ -539,6 +559,8 @@ static void read_pcap(struct ctx *ctx)
 	struct sock_fprog bpf_ops;
 	struct frame_map fm;
 	struct timeval start, end, diff;
+	bool is_out_pcap = ctx->device_out && strstr(ctx->device_out, ".pcap");
+	const struct pcap_file_ops *pcap_out_ops = pcap_ops[PCAP_OPS_RW];
 
 	bug_on(!__pcap_io);
 
@@ -548,11 +570,20 @@ static void read_pcap(struct ctx *ctx)
 		if (ctx->pcap == PCAP_OPS_MM)
 			ctx->pcap = PCAP_OPS_SG;
 	} else {
-		fd = open_or_die(ctx->device_in, O_RDONLY | O_LARGEFILE | O_NOATIME);
+		/* O_NOATIME requires privileges, in case we don't have
+		 * them, retry without them at a minor cost of updating
+		 * atime in case the fs has been mounted as such.
+		 */
+		fd = open(ctx->device_in, O_RDONLY | O_LARGEFILE | O_NOATIME);
+		if (fd < 0 && errno == EPERM)
+			fd = open_or_die(ctx->device_in, O_RDONLY | O_LARGEFILE);
+		if (fd < 0)
+			panic("Cannot open file %s! %s.\n", ctx->device_in,
+			      strerror(errno));
 	}
 
 	if (__pcap_io->init_once_pcap)
-		__pcap_io->init_once_pcap();
+		__pcap_io->init_once_pcap(false);
 
 	ret = __pcap_io->pull_fhdr_pcap(fd, &ctx->magic, &ctx->link_type);
 	if (ret)
@@ -565,7 +596,6 @@ static void read_pcap(struct ctx *ctx)
 	}
 
 	fmemset(&fm, 0, sizeof(fm));
-	fmemset(&bpf_ops, 0, sizeof(bpf_ops));
 
 	bpf_parse_rules(ctx->filter, &bpf_ops, ctx->link_type);
 	if (ctx->dump_bpf)
@@ -586,6 +616,13 @@ static void read_pcap(struct ctx *ctx)
 		}
 	}
 
+	if (is_out_pcap) {
+		ret = pcap_out_ops->push_fhdr_pcap(fdo, ctx->magic,
+						   ctx->link_type);
+		if (ret)
+			panic("Error writing pcap header!\n");
+	}
+
 	drop_privileges(ctx->enforce, ctx->uid, ctx->gid);
 
 	printf("Running! Hang up with ^C!\n\n");
@@ -618,13 +655,23 @@ static void read_pcap(struct ctx *ctx)
 		ctx->tx_bytes += fm.tp_h.tp_len;
 		ctx->tx_packets++;
 
-		show_frame_hdr(&fm, ctx->print_mode);
+		show_frame_hdr(out, fm.tp_h.tp_snaplen, ctx->link_type, &fm,
+			       ctx->print_mode, ctx->tx_packets);
 
 		dissector_entry_point(out, fm.tp_h.tp_snaplen,
-				      ctx->link_type, ctx->print_mode);
+				      ctx->link_type, ctx->print_mode,
+				      fm.s_ll.sll_protocol);
 
-		if (ctx->device_out)
+		if (is_out_pcap) {
+			size_t pcap_len = pcap_get_length(&phdr, ctx->magic);
+			int wlen = pcap_out_ops->write_pcap(fdo, &phdr,
+							    ctx->magic, out,
+							    pcap_len);
+			if (unlikely(wlen != (int)pcap_get_total_length(&phdr, ctx->magic)))
+				panic("Error writing to pcap!\n");
+		} else if (ctx->device_out) {
 			translate_pcap_to_txf(fdo, out, fm.tp_h.tp_snaplen);
+		}
 
 		if (frame_count_max != 0) {
 			if (ctx->tx_packets >= frame_count_max) {
@@ -634,8 +681,7 @@ static void read_pcap(struct ctx *ctx)
 		}
 	}
 
-	out:
-
+out:
 	bug_on(gettimeofday(&end, NULL));
 	timersub(&end, &start, &diff);
 
@@ -710,6 +756,18 @@ static int next_multi_pcap_file(struct ctx *ctx, int fd)
 	return fd;
 }
 
+static void reset_interval(struct ctx *ctx)
+{
+	if (ctx->dump_mode == DUMP_INTERVAL_TIME) {
+		interval = ctx->dump_interval;
+
+		set_itimer_interval_value(&itimer, interval, 0);
+		setitimer(ITIMER_REAL, &itimer, NULL);
+	} else {
+		interval = 0;
+	}
+}
+
 static int begin_multi_pcap_file(struct ctx *ctx)
 {
 	int fd, ret;
@@ -736,14 +794,7 @@ static int begin_multi_pcap_file(struct ctx *ctx)
 			panic("Error prepare writing pcap!\n");
 	}
 
-	if (ctx->dump_mode == DUMP_INTERVAL_TIME) {
-		interval = ctx->dump_interval;
-
-		set_itimer_interval_value(&itimer, interval, 0);
-		setitimer(ITIMER_REAL, &itimer, NULL);
-	} else {
-		interval = 0;
-	}
+	reset_interval(ctx);
 
 	return fd;
 }
@@ -802,7 +853,7 @@ static void print_pcap_file_stats(int sock, struct ctx *ctx)
 	ret = getsockopt(sock, SOL_PACKET, PACKET_STATISTICS, &kstats, &slen);
 	if (unlikely(ret))
 		panic("Cannot get packet statistics!\n");
-	
+
 	if (ctx->print_mode == PRINT_NONE) {
 		printf(".(+%u/-%u)", kstats.tp_packets - kstats.tp_drops,
 		       kstats.tp_drops);
@@ -810,20 +861,50 @@ static void print_pcap_file_stats(int sock, struct ctx *ctx)
 	}
 }
 
+static void update_pcap_next_dump(struct ctx *ctx, unsigned long snaplen, int *fd, int sock)
+{
+	if (!dump_to_pcap(ctx))
+		return;
+
+	if (ctx->dump_mode == DUMP_INTERVAL_SIZE) {
+		interval += snaplen;
+		if (interval > ctx->dump_interval) {
+			next_dump = true;
+			interval = 0;
+		}
+	}
+
+	if (sighup) {
+		if (ctx->verbose)
+			printf("SIGHUP received, prematurely rotating pcap\n");
+		sighup = 0;
+		next_dump = true;
+		reset_interval(ctx);
+	}
+
+	if (next_dump) {
+		*fd = next_multi_pcap_file(ctx, *fd);
+		next_dump = false;
+
+		if (ctx->verbose)
+			print_pcap_file_stats(sock, ctx);
+	}
+}
+
+#ifdef HAVE_TPACKET3
 static void walk_t3_block(struct block_desc *pbd, struct ctx *ctx,
 			  int sock, int *fd, unsigned long *frame_count)
 {
-	uint8_t *packet;
-	int num_pkts = pbd->h1.num_pkts, i, ret;
+	int num_pkts = pbd->h1.num_pkts, i;
 	struct tpacket3_hdr *hdr;
-	pcap_pkthdr_t phdr;
 	struct sockaddr_ll *sll;
 
 	hdr = (void *) ((uint8_t *) pbd + pbd->h1.offset_to_first_pkt);
 	sll = (void *) ((uint8_t *) hdr + TPACKET_ALIGN(sizeof(*hdr)));
 
 	for (i = 0; i < num_pkts && likely(sigint == 0); ++i) {
-		packet = ((uint8_t *) hdr + hdr->tp_mac);
+		uint8_t *packet = ((uint8_t *) hdr + hdr->tp_mac);
+		pcap_pkthdr_t phdr;
 
 		if (ctx->packet_type != -1)
 			if (ctx->packet_type != sll->sll_pkttype)
@@ -832,6 +913,8 @@ static void walk_t3_block(struct block_desc *pbd, struct ctx *ctx,
 		(*frame_count)++;
 
 		if (dump_to_pcap(ctx)) {
+			int ret;
+
 			tpacket3_hdr_to_pcap_pkthdr(hdr, sll, &phdr, ctx->magic);
 
 			ret = __pcap_io->write_pcap(*fd, &phdr, ctx->magic, packet,
@@ -840,10 +923,11 @@ static void walk_t3_block(struct block_desc *pbd, struct ctx *ctx,
 				panic("Write error to pcap!\n");
 		}
 
-		__show_frame_hdr(sll, hdr, ctx->print_mode, true);
+		__show_frame_hdr(packet, hdr->tp_snaplen, ctx->link_type, sll,
+				 hdr, ctx->print_mode, true, *frame_count);
 
 		dissector_entry_point(packet, hdr->tp_snaplen, ctx->link_type,
-				      ctx->print_mode);
+				      ctx->print_mode, sll->sll_protocol);
 next:
                 hdr = (void *) ((uint8_t *) hdr + hdr->tp_next_offset);
 		sll = (void *) ((uint8_t *) hdr + TPACKET_ALIGN(sizeof(*hdr)));
@@ -855,52 +939,25 @@ next:
 			}
 		}
 
-		if (dump_to_pcap(ctx)) {
-			if (ctx->dump_mode == DUMP_INTERVAL_SIZE) {
-				interval += hdr->tp_snaplen;
-				if (interval > ctx->dump_interval) {
-					next_dump = true;
-					interval = 0;
-				}
-			}
-
-			if (next_dump) {
-				*fd = next_multi_pcap_file(ctx, *fd);
-				next_dump = false;
-
-				if (unlikely(ctx->verbose))
-					print_pcap_file_stats(sock, ctx);
-			}
-		}
+		update_pcap_next_dump(ctx, hdr->tp_snaplen, fd, sock);
 	}
 }
+#endif /* HAVE_TPACKET3 */
 
 static void recv_only_or_dump(struct ctx *ctx)
 {
 	short ifflags = 0;
-	int sock, irq, ifindex, fd = 0, ret;
-	unsigned int size, it = 0;
+	int sock, ifindex, fd = 0, ret;
+	size_t size;
+	unsigned int it = 0;
 	struct ring rx_ring;
 	struct pollfd rx_poll;
 	struct sock_fprog bpf_ops;
 	struct timeval start, end, diff;
-	struct block_desc *pbd;
 	unsigned long frame_count = 0;
 
 	sock = pf_socket();
 
-	if (ctx->rfraw) {
-		ctx->device_trans = xstrdup(ctx->device_in);
-		xfree(ctx->device_in);
-
-		enter_rfmon_mac80211(ctx->device_trans, &ctx->device_in);
-		ctx->link_type = LINKTYPE_IEEE802_11;
-	}
-
-	fmemset(&rx_ring, 0, sizeof(rx_ring));
-	fmemset(&rx_poll, 0, sizeof(rx_poll));
-	fmemset(&bpf_ops, 0, sizeof(bpf_ops));
-
 	ifindex = device_ifindex(ctx->device_in);
 
 	size = ring_size(ctx->device_in, ctx->reserve_size);
@@ -912,21 +969,19 @@ static void recv_only_or_dump(struct ctx *ctx)
 		bpf_dump_all(&bpf_ops);
 	bpf_attach_to_sock(sock, &bpf_ops);
 
-	ret = set_sockopt_hwtimestamp(sock, ctx->device_in);
-	if (ret == 0 && ctx->verbose)
-		printf("HW timestamping enabled\n");
+	if (ctx->hwtimestamp) {
+		ret = set_sockopt_hwtimestamp(sock, ctx->device_in);
+		if (ret == 0 && ctx->verbose)
+			printf("HW timestamping enabled\n");
+	}
 
-	setup_rx_ring_layout(sock, &rx_ring, size, true, true);
-	create_rx_ring(sock, &rx_ring, ctx->verbose);
-	mmap_rx_ring(sock, &rx_ring);
-	alloc_rx_ring_frames(sock, &rx_ring);
-	bind_rx_ring(sock, &rx_ring, ifindex);
+	ring_rx_setup(&rx_ring, sock, size, ifindex, &rx_poll, is_defined(HAVE_TPACKET3), true,
+		      ctx->verbose, ctx->fanout_group, ctx->fanout_type);
 
-	prepare_polling(sock, &rx_poll);
 	dissector_init_all(ctx->print_mode);
 
 	if (ctx->cpu >= 0 && ifindex > 0) {
-		irq = device_irq_number(ctx->device_in);
+		int irq = device_irq_number(ctx->device_in);
 		device_set_irq_affinity(irq, ctx->cpu);
 
 		if (ctx->verbose)
@@ -938,7 +993,7 @@ static void recv_only_or_dump(struct ctx *ctx)
 		ifflags = device_enter_promiscuous_mode(ctx->device_in);
 
 	if (dump_to_pcap(ctx) && __pcap_io->init_once_pcap)
-		__pcap_io->init_once_pcap();
+		__pcap_io->init_once_pcap(true);
 
 	drop_privileges(ctx->enforce, ctx->uid, ctx->gid);
 
@@ -963,8 +1018,10 @@ static void recv_only_or_dump(struct ctx *ctx)
 	bug_on(gettimeofday(&start, NULL));
 
 	while (likely(sigint == 0)) {
-		while (user_may_pull_from_rx_block((pbd = (void *)
-				rx_ring.frames[it].iov_base))) {
+#ifdef HAVE_TPACKET3
+		struct block_desc *pbd;
+
+		while (user_may_pull_from_rx_block((pbd = rx_ring.frames[it].iov_base))) {
 			walk_t3_block(pbd, ctx, sock, &fd, &frame_count);
 
 			kernel_may_pull_from_rx_block(pbd);
@@ -973,6 +1030,58 @@ static void recv_only_or_dump(struct ctx *ctx)
 			if (unlikely(sigint == 1))
 				break;
 		}
+#else
+		while (user_may_pull_from_rx(rx_ring.frames[it].iov_base)) {
+			struct frame_map *hdr = rx_ring.frames[it].iov_base;
+			uint8_t *packet = ((uint8_t *) hdr) + hdr->tp_h.tp_mac;
+			pcap_pkthdr_t phdr;
+
+			if (ctx->packet_type != -1)
+				if (ctx->packet_type != hdr->s_ll.sll_pkttype)
+					goto next;
+
+			frame_count++;
+
+			if (unlikely(ring_frame_size(&rx_ring) < hdr->tp_h.tp_snaplen)) {
+				/* XXX: silently ignore for now. We used to
+				 * report them with sock_rx_net_stats()  */
+				goto next;
+			}
+
+			if (dump_to_pcap(ctx)) {
+				tpacket_hdr_to_pcap_pkthdr(&hdr->tp_h, &hdr->s_ll, &phdr, ctx->magic);
+
+				ret = __pcap_io->write_pcap(fd, &phdr, ctx->magic, packet,
+							    pcap_get_length(&phdr, ctx->magic));
+				if (unlikely(ret != (int) pcap_get_total_length(&phdr, ctx->magic)))
+					panic("Write error to pcap!\n");
+			}
+
+			show_frame_hdr(packet, hdr->tp_h.tp_snaplen,
+				       ctx->link_type, hdr, ctx->print_mode,
+				       frame_count);
+
+			dissector_entry_point(packet, hdr->tp_h.tp_snaplen,
+					      ctx->link_type, ctx->print_mode,
+					      hdr->s_ll.sll_protocol);
+
+			if (frame_count_max != 0) {
+				if (unlikely(frame_count >= frame_count_max)) {
+					sigint = 1;
+					break;
+				}
+			}
+
+next:
+			kernel_may_pull_from_rx(&hdr->tp_h);
+			it = (it + 1) % rx_ring.layout.tp_frame_nr;
+
+			if (unlikely(sigint == 1))
+				break;
+
+			update_pcap_next_dump(ctx, hdr->tp_h.tp_snaplen, &fd, sock);
+		}
+#endif /* HAVE_TPACKET3 */
 
 		ret = poll(&rx_poll, 1, -1);
 		if (unlikely(ret < 0)) {
@@ -1017,12 +1126,15 @@ static void recv_only_or_dump(struct ctx *ctx)
 static void init_ctx(struct ctx *ctx)
 {
 	memset(ctx, 0, sizeof(*ctx));
+
 	ctx->uid = getuid();
-	ctx->uid = getgid();
+	ctx->gid = getgid();
 
 	ctx->cpu = -1;
 	ctx->packet_type = -1;
 
+	ctx->fanout_type = PACKET_FANOUT_ROLLOVER;
+
 	ctx->magic = ORIGINAL_TCPDUMP_MAGIC;
 	ctx->print_mode = PRINT_NORM;
 	ctx->pcap = PCAP_OPS_SG;
@@ -1032,6 +1144,7 @@ static void init_ctx(struct ctx *ctx)
 
 	ctx->promiscuous = true;
 	ctx->randomize = false;
+	ctx->hwtimestamp = true;
 }
 
 static void destroy_ctx(struct ctx *ctx)
@@ -1045,12 +1158,15 @@ static void destroy_ctx(struct ctx *ctx)
 
 static void __noreturn help(void)
 {
-	printf("\nnetsniff-ng %s, the packet sniffing beast\n", VERSION_STRING);
+	printf("netsniff-ng %s, the packet sniffing beast\n", VERSION_STRING);
 	puts("http://www.netsniff-ng.org\n\n"
 	     "Usage: netsniff-ng [options] [filter-expression]\n"
 	     "Options:\n"
 	     "  -i|-d|--dev|--in <dev|pcap|->  Input source as netdev, pcap or pcap stdin\n"
 	     "  -o|--out <dev|pcap|dir|cfg|->  Output sink as netdev, pcap, directory, trafgen, or stdout\n"
+	     "  -C|--fanout-group <id>         Join packet fanout group\n"
+	     "  -K|--fanout-type <type>        Apply fanout discipline: hash|lb|cpu|rnd|roll|qm\n"
+	     "  -L|--fanout-opts <opts>        Additional fanout options: defrag|roll\n"
 	     "  -f|--filter <bpf-file|expr>    Use BPF filter file from bpfc or tcpdump-like expression\n"
 	     "  -t|--type <type>               Filter for: host|broadcast|multicast|others|outgoing\n"
 	     "  -F|--interval <size|time>      Dump interval if -o is a dir: <num>KiB/MiB/GiB/s/sec/min/hrs\n"
@@ -1063,6 +1179,7 @@ static void __noreturn help(void)
 	     "  -r|--rand                      Randomize packet forwarding order (dev->dev)\n"
 	     "  -M|--no-promisc                No promiscuous mode for netdev\n"
 	     "  -A|--no-sock-mem               Don't tune core socket memory\n"
+	     "  -N|--no-hwtimestamp            Disable hardware time stamping\n"
 	     "  -m|--mmap                      Mmap(2) pcap file I/O, e.g. for replaying pcaps\n"
 	     "  -G|--sg                        Scatter/gather pcap file I/O\n"
 	     "  -c|--clrw                      Use slower read(2)/write(2) I/O\n"
@@ -1087,37 +1204,24 @@ static void __noreturn help(void)
 	     "  netsniff-ng --in wlan0 --rfraw --out dump.pcap --silent --bind-cpu 0\n"
 	     "  netsniff-ng --in dump.pcap --mmap --out eth0 -k1000 --silent --bind-cpu 0\n"
 	     "  netsniff-ng --in dump.pcap --out dump.cfg --silent --bind-cpu 0\n"
+	     "  netsniff-ng --in dump.pcap --out dump2.pcap --silent tcp\n"
 	     "  netsniff-ng --in eth0 --out eth1 --silent --bind-cpu 0 -J --type host\n"
 	     "  netsniff-ng --in eth1 --out /opt/probe/ -s -m --interval 100MiB -b 0\n"
 	     "  netsniff-ng --in vlan0 --out dump.pcap -c -u `id -u bob` -g `id -g bob`\n"
 	     "  netsniff-ng --in any --filter http.bpf --jumbo-support --ascii -V\n\n"
 	     "Note:\n"
 	     "  For introducing bit errors, delays with random variation and more\n"
-	     "  while replaying pcaps, make use of tc(8) with its disciplines (e.g. netem).\n\n"
-	     "Please report bugs to <bugs@netsniff-ng.org>\n"
-	     "Copyright (C) 2009-2013 Daniel Borkmann <dborkma@tik.ee.ethz.ch>\n"
-	     "Copyright (C) 2009-2012 Emmanuel Roullit <emmanuel.roullit@gmail.com>\n"
-	     "Copyright (C) 2012 Markus Amend <markus@netsniff-ng.org>\n"
-	     "Swiss federal institute of technology (ETH Zurich)\n"
-	     "License: GNU GPL version 2.0\n"
-	     "This is free software: you are free to change and redistribute it.\n"
-	     "There is NO WARRANTY, to the extent permitted by law.\n");
+	     "  while replaying pcaps, make use of tc(8) with its disciplines (e.g. netem).\n");
+	puts(copyright);
 	die();
 }
 
 static void __noreturn version(void)
 {
-	printf("\nnetsniff-ng %s, Git id: %s\n", VERSION_LONG, GITVERSION);
+	printf("netsniff-ng %s, Git id: %s\n", VERSION_LONG, GITVERSION);
 	puts("the packet sniffing beast\n"
-	     "http://www.netsniff-ng.org\n\n"
-	     "Please report bugs to <bugs@netsniff-ng.org>\n"
-	     "Copyright (C) 2009-2013 Daniel Borkmann <dborkma@tik.ee.ethz.ch>\n"
-	     "Copyright (C) 2009-2012 Emmanuel Roullit <emmanuel.roullit@gmail.com>\n"
-	     "Copyright (C) 2012 Markus Amend <markus@netsniff-ng.org>\n"
-	     "Swiss federal institute of technology (ETH Zurich)\n"
-	     "License: GNU GPL version 2.0\n"
-	     "This is free software: you are free to change and redistribute it.\n"
-	     "There is NO WARRANTY, to the extent permitted by law.\n");
+	     "http://www.netsniff-ng.org\n");
+	puts(copyright);
 	die();
 }
 
@@ -1146,7 +1250,6 @@ int main(int argc, char **argv)
 			ctx.prefix = xstrdup(optarg);
 			break;
 		case 'R':
-			ctx.link_type = LINKTYPE_IEEE802_11;
 			ctx.rfraw = 1;
 			break;
 		case 'r':
@@ -1165,6 +1268,9 @@ int main(int argc, char **argv)
 		case 'M':
 			ctx.promiscuous = false;
 			break;
+		case 'N':
+			ctx.hwtimestamp = false;
+			break;
 		case 'A':
 			setsockmem = false;
 			break;
@@ -1176,6 +1282,36 @@ int main(int argc, char **argv)
 			ctx.gid = strtoul(optarg, NULL, 0);
 			ctx.enforce = true;
 			break;
+		case 'C':
+			ctx.fanout_group = strtoul(optarg, NULL, 0);
+			if (ctx.fanout_group == 0)
+				panic("Non-zero fanout group id required!\n");
+			break;
+		case 'K':
+			if (!strncmp(optarg, "hash", strlen("hash")))
+				ctx.fanout_type = PACKET_FANOUT_HASH;
+			else if (!strncmp(optarg, "lb", strlen("lb")) ||
+				 !strncmp(optarg, "rr", strlen("rr")))
+				ctx.fanout_type = PACKET_FANOUT_LB;
+			else if (!strncmp(optarg, "cpu", strlen("cpu")))
+				ctx.fanout_type = PACKET_FANOUT_CPU;
+			else if (!strncmp(optarg, "rnd", strlen("rnd")))
+				ctx.fanout_type = PACKET_FANOUT_RND;
+			else if (!strncmp(optarg, "roll", strlen("roll")))
+				ctx.fanout_type = PACKET_FANOUT_ROLLOVER;
+			else if (!strncmp(optarg, "qm", strlen("qm")))
+				ctx.fanout_type = PACKET_FANOUT_QM;
+			else
+				panic("Unkown fanout type!\n");
+			break;
+		case 'L':
+			if (!strncmp(optarg, "defrag", strlen("defrag")))
+				ctx.fanout_type |= PACKET_FANOUT_FLAG_DEFRAG;
+			else if (!strncmp(optarg, "roll", strlen("roll")))
+				ctx.fanout_type |= PACKET_FANOUT_FLAG_ROLLOVER;
+			else
+				panic("Unkown fanout option!\n");
+			break;
 		case 't':
 			if (!strncmp(optarg, "host", strlen("host")))
 				ctx.packet_type = PACKET_HOST;
@@ -1192,8 +1328,6 @@ int main(int argc, char **argv)
 			break;
 		case 'S':
 			ptr = optarg;
-			ctx.reserve_size = 0;
-
 			for (j = i = strlen(optarg); i > 0; --i) {
 				if (!isdigit(optarg[j - i]))
 					break;
@@ -1260,8 +1394,6 @@ int main(int argc, char **argv)
 			break;
 		case 'F':
 			ptr = optarg;
-			ctx.dump_interval = 0;
-
 			for (j = i = strlen(optarg); i > 0; --i) {
 				if (!isdigit(optarg[j - i]))
 					break;
@@ -1296,7 +1428,7 @@ int main(int argc, char **argv)
 			ctx.dump_interval *= strtoul(optarg, NULL, 0);
 			break;
 		case 'V':
-			ctx.verbose = 1;
+			ctx.verbose = true;
 			break;
 		case 'B':
 			ctx.dump_bpf = true;
@@ -1352,7 +1484,7 @@ int main(int argc, char **argv)
 			size_t alen = strlen(argv[i]) + 2;
 			size_t flen = ctx.filter ? strlen(ctx.filter) : 0;
 
-			ctx.filter = xrealloc(ctx.filter, 1, flen + alen);
+			ctx.filter = xrealloc(ctx.filter, flen + alen);
 			ret = slprintf(ctx.filter + offset, strlen(argv[i]) + 2, "%s ", argv[i]);
 			if (ret < 0)
 				panic("Cannot concatenate filter string!\n");
@@ -1376,10 +1508,12 @@ int main(int argc, char **argv)
 		set_sched_status(SCHED_FIFO, sched_get_priority_max(SCHED_FIFO));
 	}
 
-	if (ctx.device_in && (device_mtu(ctx.device_in) ||
-	    !strncmp("any", ctx.device_in, strlen(ctx.device_in)))) {
-		if (!ctx.rfraw)
-			ctx.link_type = pcap_devtype_to_linktype(ctx.device_in);
+	if (device_mtu(ctx.device_in) || !strncmp("any", ctx.device_in, strlen(ctx.device_in))) {
+		if (ctx.rfraw)
+			setup_rfmon_mac80211_dev(&ctx, &ctx.device_in);
+
+		ctx.link_type = pcap_devtype_to_linktype(ctx.device_in);
+
 		if (!ctx.device_out) {
 			ctx.dump = 0;
 			main_loop = recv_only_or_dump;
