@@ -9,8 +9,8 @@
 #include <netinet/in.h>    /* for ntohs() */
 
 #include "proto.h"
+#include "protos.h"
 #include "csum.h"
-#include "dissector_eth.h"
 #include "pkt_buff.h"
 #include "built_in.h"
 
